/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2017 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

var juju = { components: {} }; // eslint-disable-line no-unused-vars

chai.config.includeStack = true;
chai.config.truncateThreshold = 0;

describe('CreatePaymentUser', function () {
  let acl, getCountries, onUserCreated, refs;

  beforeAll(function (done) {
    // By loading this file it adds the component to the juju components.
    YUI().use('create-payment-user', function () {
      done();
    });
  });

  beforeEach(() => {
    acl = { isReadOnly: sinon.stub().returns(false) };
    onUserCreated = sinon.stub();
    getCountries = sinon.stub();
    refs = {
      emailAddress: {
        getValue: sinon.stub().returns('spinach@example.com')
      },
      userAddress: {
        getValue: sinon.stub().returns({
          name: 'Geoffrey Spinach',
          line1: '10 Maple St',
          line2: '',
          city: 'Sasquatch',
          state: 'Bunnyhug',
          postcode: '90210',
          countryCode: 'CA',
          phones: ['12341234']
        })
      },
      cardForm: {
        getValue: sinon.stub().returns({
          card: { card: 'value' },
          name: 'Mr Geoffrey Spinach'
        })
      }
    };
  });

  it('can display a personal form', function () {
    const addNotification = sinon.stub();
    const validateForm = sinon.stub();
    const createCardElement = sinon.stub();
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.CreatePaymentUser, {
      acl: acl,
      addNotification: addNotification,
      createCardElement: createCardElement,
      createToken: sinon.stub(),
      createUser: sinon.stub(),
      getCountries: getCountries,
      onUserCreated: onUserCreated,
      username: 'spinach',
      validateForm: validateForm }), true);
    const instance = renderer.getMountedInstance();
    const output = renderer.getRenderOutput();
    const options = output.props.children.props.children[0].props.children[0].props.children;
    const expected = React.createElement(
      'div',
      { className: 'create-payment-user' },
      React.createElement(
        'form',
        { className: 'create-payment-user__form' },
        React.createElement(
          'div',
          { className: 'create-payment-user__form-content' },
          React.createElement(
            'ul',
            { className: 'create-payment-user__form-type' },
            React.createElement(
              'li',
              { className: 'create-payment-user__form-type-option' },
              React.createElement(
                'label',
                { htmlFor: 'personal' },
                React.createElement('input', { checked: true,
                  id: 'personal',
                  name: 'formType',
                  onChange: options[0].props.children.props.children[0].props.onChange,
                  type: 'radio' }),
                'Personal use'
              )
            ),
            React.createElement(
              'li',
              { className: 'create-payment-user__form-type-option' },
              React.createElement(
                'label',
                { htmlFor: 'business' },
                React.createElement('input', { checked: false,
                  id: 'business',
                  name: 'formType',
                  onChange: options[1].props.children.props.children[0].props.onChange,
                  type: 'radio' }),
                'Business use'
              )
            )
          ),
          null,
          React.createElement(
            'h2',
            { className: 'create-payment-user__title' },
            'Name and address'
          ),
          null,
          React.createElement(juju.components.GenericInput, {
            disabled: false,
            label: 'Email address',
            ref: 'emailAddress',
            required: true,
            validate: [{
              regex: /\S+/,
              error: 'This field is required.'
            }] }),
          React.createElement(juju.components.AddressForm, {
            addNotification: addNotification,
            disabled: false,
            getCountries: getCountries,
            ref: 'userAddress',
            validateForm: validateForm }),
          React.createElement(
            'h2',
            { className: 'create-payment-user__title' },
            'Payment information'
          ),
          React.createElement(juju.components.CardForm, {
            acl: acl,
            createCardElement: createCardElement,
            ref: 'cardForm',
            validateForm: validateForm }),
          React.createElement(
            'label',
            { htmlFor: 'cardAddressSame' },
            React.createElement('input', { checked: true,
              id: 'cardAddressSame',
              name: 'cardAddressSame',
              onChange: instance._handleCardSameChange,
              ref: 'cardAddressSame',
              type: 'checkbox' }),
            'Credit or debit card address is the same as above'
          ),
          React.createElement(
            'label',
            { htmlFor: 'billingAddressSame' },
            React.createElement('input', { checked: true,
              id: 'billingAddressSame',
              name: 'billingAddressSame',
              onChange: instance._handleBillingSameChange,
              ref: 'billingAddressSame',
              type: 'checkbox' }),
            'Billing address is the same as above'
          ),
          null,
          null
        ),
        React.createElement(
          'div',
          { className: 'create-payment-user__add' },
          React.createElement(juju.components.GenericButton, {
            action: instance._handleAddUser,
            disabled: false,
            type: 'inline-neutral',
            title: 'Add payment details' })
        )
      )
    );
    expect(output).toEqualJSX(expected);
  });

  it('can display a business form', function () {
    const addNotification = sinon.stub();
    const validateForm = sinon.stub();
    const createCardElement = sinon.stub();
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.CreatePaymentUser, {
      acl: acl,
      addNotification: addNotification,
      createCardElement: createCardElement,
      createToken: sinon.stub(),
      createUser: sinon.stub(),
      getCountries: getCountries,
      onUserCreated: onUserCreated,
      username: 'spinach',
      validateForm: validateForm }), true);
    const instance = renderer.getMountedInstance();
    let output = renderer.getRenderOutput();
    output.props.children.props.children[0].props.children[0].props.children[1].props.children.props.children[0].props.onChange();
    output = renderer.getRenderOutput();
    const options = output.props.children.props.children[0].props.children[0].props.children;
    const expected = React.createElement(
      'div',
      { className: 'create-payment-user' },
      React.createElement(
        'form',
        { className: 'create-payment-user__form' },
        React.createElement(
          'div',
          { className: 'create-payment-user__form-content' },
          React.createElement(
            'ul',
            { className: 'create-payment-user__form-type' },
            React.createElement(
              'li',
              { className: 'create-payment-user__form-type-option' },
              React.createElement(
                'label',
                { htmlFor: 'personal' },
                React.createElement('input', { checked: false,
                  id: 'personal',
                  name: 'formType',
                  onChange: options[0].props.children.props.children[0].props.onChange,
                  type: 'radio' }),
                'Personal use'
              )
            ),
            React.createElement(
              'li',
              { className: 'create-payment-user__form-type-option' },
              React.createElement(
                'label',
                { htmlFor: 'business' },
                React.createElement('input', { checked: true,
                  id: 'business',
                  name: 'formType',
                  onChange: options[1].props.children.props.children[0].props.onChange,
                  type: 'radio' }),
                'Business use'
              )
            )
          ),
          React.createElement(
            'div',
            { className: 'create-payment-user__vat' },
            React.createElement(juju.components.GenericInput, {
              disabled: false,
              label: 'VAT number (optional)',
              ref: 'VATNumber',
              required: false })
          ),
          React.createElement(
            'h2',
            { className: 'create-payment-user__title' },
            'Name and address'
          ),
          React.createElement(juju.components.GenericInput, {
            disabled: false,
            label: 'Business name',
            ref: 'businessName',
            required: true,
            validate: [{
              regex: /\S+/,
              error: 'This field is required.'
            }] }),
          React.createElement(juju.components.GenericInput, {
            disabled: false,
            label: 'Email address',
            ref: 'emailAddress',
            required: true,
            validate: [{
              regex: /\S+/,
              error: 'This field is required.'
            }] }),
          React.createElement(juju.components.AddressForm, {
            addNotification: addNotification,
            disabled: false,
            getCountries: getCountries,
            ref: 'userAddress',
            validateForm: validateForm }),
          React.createElement(
            'h2',
            { className: 'create-payment-user__title' },
            'Payment information'
          ),
          React.createElement(juju.components.CardForm, {
            acl: acl,
            createCardElement: createCardElement,
            ref: 'cardForm',
            validateForm: validateForm }),
          React.createElement(
            'label',
            { htmlFor: 'cardAddressSame' },
            React.createElement('input', { checked: true,
              id: 'cardAddressSame',
              name: 'cardAddressSame',
              onChange: instance._handleCardSameChange,
              ref: 'cardAddressSame',
              type: 'checkbox' }),
            'Credit or debit card address is the same as above'
          ),
          React.createElement(
            'label',
            { htmlFor: 'billingAddressSame' },
            React.createElement('input', { checked: true,
              id: 'billingAddressSame',
              name: 'billingAddressSame',
              onChange: instance._handleBillingSameChange,
              ref: 'billingAddressSame',
              type: 'checkbox' }),
            'Billing address is the same as above'
          ),
          null,
          null
        ),
        React.createElement(
          'div',
          { className: 'create-payment-user__add' },
          React.createElement(juju.components.GenericButton, {
            action: instance._handleAddUser,
            disabled: false,
            type: 'inline-neutral',
            title: 'Add payment details' })
        )
      )
    );
    expect(output).toEqualJSX(expected);
  });

  it('can display card and billing address fields', function () {
    const addNotification = sinon.stub();
    const validateForm = sinon.stub();
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.CreatePaymentUser, {
      acl: acl,
      addNotification: addNotification,
      createCardElement: sinon.stub(),
      createToken: sinon.stub(),
      createUser: sinon.stub(),
      getCountries: getCountries,
      onUserCreated: onUserCreated,
      username: 'spinach',
      validateForm: validateForm }), true);
    let output = renderer.getRenderOutput();
    let formContent = output.props.children.props.children[0].props.children;
    formContent[8].props.children[0].props.onChange({ currentTarget: { checked: false } });
    formContent[9].props.children[0].props.onChange({ currentTarget: { checked: false } });
    output = renderer.getRenderOutput();
    formContent = output.props.children.props.children[0].props.children;
    expect(formContent[10]).toEqualJSX(React.createElement(
      'div',
      null,
      React.createElement(
        'h2',
        { className: 'create-payment-user__title' },
        'Card address'
      ),
      React.createElement(juju.components.AddressForm, {
        addNotification: addNotification,
        disabled: false,
        getCountries: getCountries,
        ref: 'cardAddress',
        validateForm: validateForm })
    ));
    expect(formContent[11]).toEqualJSX(React.createElement(
      'div',
      null,
      React.createElement(
        'h2',
        { className: 'create-payment-user__title' },
        'Billing address'
      ),
      React.createElement(juju.components.AddressForm, {
        addNotification: addNotification,
        disabled: false,
        getCountries: getCountries,
        ref: 'billingAddress',
        validateForm: validateForm })
    ));
  });

  it('can abort requests when unmounting', function () {
    const abort = sinon.stub();
    const createToken = sinon.stub().returns({ abort: abort });
    const component = jsTestUtils.shallowRender(React.createElement(juju.components.CreatePaymentUser, {
      acl: acl,
      addNotification: sinon.stub(),
      createCardElement: sinon.stub(),
      createToken: createToken,
      createUser: sinon.stub(),
      getCountries: getCountries,
      onUserCreated: onUserCreated,
      username: 'spinach',
      validateForm: sinon.stub().returns(true) }), true);
    const instance = component.getMountedInstance();
    instance.refs = refs;
    const output = component.getRenderOutput();
    output.props.children.props.children[1].props.children.props.action();
    component.unmount();
    assert.equal(abort.callCount, 1);
  });

  it('does not add the user if there is a validation error', function () {
    const createToken = sinon.stub();
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.CreatePaymentUser, {
      acl: acl,
      addNotification: sinon.stub(),
      createCardElement: sinon.stub(),
      createToken: createToken,
      createUser: sinon.stub(),
      getCountries: getCountries,
      onUserCreated: onUserCreated,
      username: 'spinach',
      validateForm: sinon.stub().returns(false) }), true);
    const output = renderer.getRenderOutput();
    output.props.children.props.children[1].props.children.props.action();
    assert.equal(createToken.callCount, 0);
  });

  it('can create the token using the correct data', function () {
    const createToken = sinon.stub();
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.CreatePaymentUser, {
      acl: acl,
      addNotification: sinon.stub(),
      createCardElement: sinon.stub(),
      createToken: createToken,
      createUser: sinon.stub(),
      getCountries: getCountries,
      onUserCreated: onUserCreated,
      username: 'spinach',
      validateForm: sinon.stub().returns(true) }), true);
    const instance = renderer.getMountedInstance();
    instance.refs = refs;
    const output = renderer.getRenderOutput();
    output.props.children.props.children[1].props.children.props.action();
    assert.equal(createToken.callCount, 1);
    assert.deepEqual(createToken.args[0][0], { card: 'value' });
    assert.deepEqual(createToken.args[0][1], {
      name: 'Mr Geoffrey Spinach',
      addressLine1: '10 Maple St',
      addressLine2: '',
      addressCity: 'Sasquatch',
      addressState: 'Bunnyhug',
      addressZip: '90210',
      addressCountry: 'CA'
    });
  });

  it('can create the token using the card address', function () {
    const createToken = sinon.stub();
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.CreatePaymentUser, {
      acl: acl,
      addNotification: sinon.stub(),
      createCardElement: sinon.stub(),
      createToken: createToken,
      createUser: sinon.stub(),
      getCountries: getCountries,
      onUserCreated: onUserCreated,
      username: 'spinach',
      validateForm: sinon.stub().returns(true) }), true);
    const instance = renderer.getMountedInstance();
    refs.cardAddress = {
      getValue: sinon.stub().returns({
        name: 'Bruce Dundee',
        line1: '9 Kangaroo St',
        line2: '',
        city: 'Snake',
        state: 'Spider',
        postcode: '9000',
        countryCode: 'AU',
        phones: ['00001111']
      })
    };
    instance.refs = refs;
    let output = renderer.getRenderOutput();
    let formContent = output.props.children.props.children[0].props.children;
    formContent[8].props.children[0].props.onChange({ currentTarget: { checked: false } });
    output = renderer.getRenderOutput();
    output.props.children.props.children[1].props.children.props.action();
    assert.equal(createToken.callCount, 1);
    assert.deepEqual(createToken.args[0][0], { card: 'value' });
    assert.deepEqual(createToken.args[0][1], {
      name: 'Mr Geoffrey Spinach',
      addressLine1: '9 Kangaroo St',
      addressLine2: '',
      addressCity: 'Snake',
      addressState: 'Spider',
      addressZip: '9000',
      addressCountry: 'AU'
    });
  });

  it('can handle errors when trying to create the token', function () {
    const addNotification = sinon.stub();
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.CreatePaymentUser, {
      acl: acl,
      addNotification: addNotification,
      createCardElement: sinon.stub(),
      createToken: sinon.stub().callsArgWith(2, 'Uh oh!', null),
      createUser: sinon.stub(),
      getCountries: getCountries,
      onUserCreated: onUserCreated,
      username: 'spinach',
      validateForm: sinon.stub().returns(true) }), true);
    const instance = renderer.getMountedInstance();
    instance.refs = refs;
    const output = renderer.getRenderOutput();
    output.props.children.props.children[1].props.children.props.action();
    assert.equal(addNotification.callCount, 1);
    assert.deepEqual(addNotification.args[0][0], {
      title: 'Could not create Stripe token',
      message: 'Could not create Stripe token: Uh oh!',
      level: 'error'
    });
  });

  it('can create the user using the correct data', function () {
    const createUser = sinon.stub();
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.CreatePaymentUser, {
      acl: acl,
      addNotification: sinon.stub(),
      createCardElement: sinon.stub(),
      createToken: sinon.stub().callsArgWith(2, null, { id: 'token_123' }),
      createUser: createUser,
      getCountries: getCountries,
      onUserCreated: onUserCreated,
      username: 'spinach',
      validateForm: sinon.stub().returns(true) }), true);
    const instance = renderer.getMountedInstance();
    instance.refs = refs;
    const output = renderer.getRenderOutput();
    output.props.children.props.children[1].props.children.props.action();
    assert.equal(createUser.callCount, 1);
    assert.deepEqual(createUser.args[0][0], {
      name: 'Geoffrey Spinach',
      email: 'spinach@example.com',
      addresses: [{
        name: 'Geoffrey Spinach',
        line1: '10 Maple St',
        line2: '',
        city: 'Sasquatch',
        state: 'Bunnyhug',
        postcode: '90210',
        countryCode: 'CA',
        phones: ['12341234']
      }],
      vat: null,
      business: false,
      businessName: null,
      billingAddresses: [{
        name: 'Geoffrey Spinach',
        line1: '10 Maple St',
        line2: '',
        city: 'Sasquatch',
        state: 'Bunnyhug',
        postcode: '90210',
        countryCode: 'CA',
        phones: ['12341234']
      }],
      token: 'token_123',
      paymentMethodName: 'Default'
    });
  });

  it('can create a business user using the correct data', function () {
    const createUser = sinon.stub();
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.CreatePaymentUser, {
      acl: acl,
      addNotification: sinon.stub(),
      createCardElement: sinon.stub(),
      createToken: sinon.stub().callsArgWith(2, null, { id: 'token_123' }),
      createUser: createUser,
      getCountries: getCountries,
      onUserCreated: onUserCreated,
      username: 'spinach',
      validateForm: sinon.stub().returns(true) }), true);
    const instance = renderer.getMountedInstance();
    const extraRefs = {
      VATNumber: {
        getValue: sinon.stub().returns('vat23')
      },
      businessName: {
        getValue: sinon.stub().returns('Tuques LTD')
      }
    };
    instance.refs = Object.assign(refs, extraRefs);
    instance.refs = refs;
    let output = renderer.getRenderOutput();
    output.props.children.props.children[0].props.children[0].props.children[1].props.children.props.children[0].props.onChange();
    output = renderer.getRenderOutput();
    output.props.children.props.children[1].props.children.props.action();
    assert.equal(createUser.callCount, 1);
    const args = createUser.args[0][0];
    assert.equal(args.business, true);
    assert.equal(args.businessName, 'Tuques LTD');
    assert.equal(args.vat, 'vat23');
  });

  it('can create the user with a different billing address', function () {
    const createUser = sinon.stub();
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.CreatePaymentUser, {
      acl: acl,
      addNotification: sinon.stub(),
      createCardElement: sinon.stub(),
      createToken: sinon.stub().callsArgWith(2, null, { id: 'token_123' }),
      createUser: createUser,
      getCountries: getCountries,
      onUserCreated: onUserCreated,
      username: 'spinach',
      validateForm: sinon.stub().returns(true) }), true);
    const instance = renderer.getMountedInstance();
    refs.billingAddress = {
      getValue: sinon.stub().returns({
        name: 'Bruce Dundee',
        line1: '9 Kangaroo St',
        line2: '',
        city: 'Snake',
        state: 'Spider',
        postcode: '9000',
        countryCode: 'AU',
        phones: ['00001111']
      })
    };
    instance.refs = refs;
    let output = renderer.getRenderOutput();
    let formContent = output.props.children.props.children[0].props.children;
    formContent[9].props.children[0].props.onChange({ currentTarget: { checked: false } });
    output = renderer.getRenderOutput();
    output.props.children.props.children[1].props.children.props.action();
    assert.equal(createUser.callCount, 1);
    assert.deepEqual(createUser.args[0][0], {
      name: 'Geoffrey Spinach',
      email: 'spinach@example.com',
      addresses: [{
        name: 'Geoffrey Spinach',
        line1: '10 Maple St',
        line2: '',
        city: 'Sasquatch',
        state: 'Bunnyhug',
        postcode: '90210',
        countryCode: 'CA',
        phones: ['12341234']
      }],
      vat: null,
      business: false,
      businessName: null,
      billingAddresses: [{
        name: 'Bruce Dundee',
        line1: '9 Kangaroo St',
        line2: '',
        city: 'Snake',
        state: 'Spider',
        postcode: '9000',
        countryCode: 'AU',
        phones: ['00001111']
      }],
      token: 'token_123',
      paymentMethodName: 'Default'
    });
  });

  it('can handle errors when trying to create the user', function () {
    const addNotification = sinon.stub();
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.CreatePaymentUser, {
      acl: acl,
      addNotification: addNotification,
      createCardElement: sinon.stub(),
      createToken: sinon.stub().callsArgWith(2, null, { id: 'token_123' }),
      createUser: sinon.stub().callsArgWith(1, 'Uh oh!', null),
      getCountries: getCountries,
      onUserCreated: onUserCreated,
      username: 'spinach',
      validateForm: sinon.stub().returns(true) }), true);
    const instance = renderer.getMountedInstance();
    instance.refs = refs;
    const output = renderer.getRenderOutput();
    output.props.children.props.children[1].props.children.props.action();
    assert.equal(addNotification.callCount, 1);
    assert.deepEqual(addNotification.args[0][0], {
      title: 'Could not create a payment user',
      message: 'Could not create a payment user: Uh oh!',
      level: 'error'
    });
  });

  it('reloads the user after the user is created', function () {
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.CreatePaymentUser, {
      acl: acl,
      addNotification: sinon.stub(),
      createCardElement: sinon.stub(),
      createToken: sinon.stub().callsArgWith(2, null, { id: 'token_123' }),
      createUser: sinon.stub().callsArgWith(1, null, null),
      getCountries: getCountries,
      onUserCreated: onUserCreated,
      username: 'spinach',
      validateForm: sinon.stub().returns(true) }), true);
    const instance = renderer.getMountedInstance();
    instance.refs = refs;
    const output = renderer.getRenderOutput();
    output.props.children.props.children[1].props.children.props.action();
    assert.equal(onUserCreated.callCount, 1);
  });
});