/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2017 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

var juju = { components: {} }; // eslint-disable-line no-unused-vars

describe('ModalSortcuts', function () {

  const keybindings = {
    test: { label: 'test', help: 'a test item' },
    test2: { label: 'test2', help: 'a second test item' }
  };

  beforeAll(function (done) {
    // By loading this file it adds the component to the juju components.
    YUI().use('modal-shortcuts', function () {
      done();
    });
  });

  function visibleRender(hide = sinon.stub()) {
    return React.createElement(
      'div',
      { className: 'modal' },
      React.createElement(
        'div',
        { className: 'twelve-col no-margin-bottom' },
        React.createElement(
          'h2',
          { className: 'bordered' },
          'Keyboard Shortcuts'
        ),
        React.createElement(
          'span',
          { className: 'close', tabIndex: '0', role: 'button',
            onClick: hide },
          React.createElement(juju.components.SvgIcon, { name: 'close_16',
            size: '16' })
        )
      ),
      React.createElement(
        'div',
        { className: 'twelve-col' },
        React.createElement(
          'div',
          { className: 'content' },
          React.createElement(
            'div',
            { key: 'test' },
            React.createElement(
              'div',
              { className: 'two-col' },
              'test'
            ),
            React.createElement(
              'div',
              { className: 'four-col last-col' },
              'a test item'
            )
          ),
          React.createElement(
            'div',
            { key: 'test2' },
            React.createElement(
              'div',
              { className: 'two-col' },
              'test2'
            ),
            React.createElement(
              'div',
              { className: 'four-col last-col' },
              'a second test item'
            )
          )
        )
      ),
      React.createElement(
        'div',
        { className: 'twelve-col' },
        React.createElement(
          'div',
          { className: 'content' },
          'Juju GUI version 1.2.3'
        )
      )
    );
  }

  it('renders', function () {
    const close = sinon.stub();
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.ModalShortcuts, {
      closeModal: close,
      guiVersion: '1.2.3',
      keybindings: keybindings }), true);
    let output = renderer.getRenderOutput();
    let expected = visibleRender(close);
    expect(output).toEqualJSX(expected);
  });
});